import { Injectable } from '@angular/core';
import { Subject } from 'rxjs';
import { DataService } from './data.service';
import { Song } from '../classes/entities';

/**
 * The audio service handles all logic that has to do with the audio player.
 */
@Injectable({
  providedIn: 'root'
})
export class AudioService {

  /**
   * The isCurrentlyPlaying property is equal to the current playstate of the player.
   */
  get isCurrentlyPlaying() {
    return !this.player.paused;
  }

  constructor(private data: DataService) {
    // Make sure that the currentTime property is always up to date.
    this.player.addEventListener('timeupdate', () => {
      this.currentTime = this.player.currentTime;
    });

    // Make sure that the currentSong property equals the current song id.
    this.currentSong$.subscribe(id => this.currentSong = id);
  }

  player = new Audio();

  currentSong$ = new Subject<number>();
  currentSong: number;
  currentTime: number;

  // Use audio.time to set the time
  set time(value: number) {
    this.player.currentTime = value;
  }

  /**
   * The play function sets the playstate of the player to playing.
   */
  play() {
    this.player.play();
  }

/**
 * The pause function sets the playstate of the player to paused.
 */
  pause() {
    this.player.pause();
  }


  /**
   * This function will format the number of seconds to a nices human-readable format.
   * @param input The number of seconds.
   * @returns a string like: '01:34', standing for 1 minute and 34 seconds.
   */
  formatTime(input: number): string {
    const minutes = Math.floor(input / 60);
    const seconds = Math.floor(input % 60);

    return this.addZero(minutes) + ':' + this.addZero(seconds);
  }


  /**
   * This function prepends a 0 when the number is smaller than 10.
   * @param input The number
   * @returns a string with a 0 prepended if the number is smaller than 10.
   */
  addZero(input: number): string {
    if (input < 10) {
      return '0' + String(input);
    } else {
      return String(input);
    }
  }

  /**
   * The setSong function set the song for the player.
   * @param song The song object
   */
  setSong(song: Song) {
    this.player.src = this.data.apiUrl + '/play/' + song.path;
    this.currentSong$.next(song.id);
  }
}
