import { Injectable } from '@angular/core';
import axios, { AxiosResponse } from 'axios';
import { AxiosInstance } from 'axios';

export interface ApiData {
  result: any;
}

/**
 * The DataService handles all data traffic to and from the API.
 * @example Get a song by id.
 * // returns something like {result: {id: 0, name: "Thunderstruck", artist: "AC/DC"}}
 * DataService.getSong(0);
 */
@Injectable({
  providedIn: 'root'
})
export class DataService {

  // This variable could even be moved elsewhere.
  public apiUrl = 'http://localhost:673';

  /** This variable will store the axios http client. */
  private axiosInstance: AxiosInstance;

  constructor() {
    this.axiosInstance = axios.create();
  }
  /**
   * @param type The type of the thing you want to get. The type is a string containing either 'artist', 'album' or 'song'.
   * @param id The id of the artist. It can be a number representing the id, multiple numbers seperated by commas or the string 'all'.
   * @returns A Promise<AxiosResponse<object>> containing the returned data from the API.
   */
  get(type: string, id: string | number): Promise<AxiosResponse<ApiData>> {
    return this.axiosInstance.get(`${this.apiUrl}/get/${type}/${id}`);
  }

  getDuration(path: string): Promise<AxiosResponse<ApiData>> {
    return this.axiosInstance.get(`${this.apiUrl}/duration/${path}`);
  }

  /**
   * @param id The id of the artist. It can be a number representing the id, multiple numbers seperated by commas or the string 'all'.
   * @returns A Promise<AxiosResponse<object>> containing the returned data from the API.
   */
  getSong(id: number | string): Promise<AxiosResponse<ApiData>> {
    return this.get('song', id);
  }

  /**
   * @param id The id of the artist. It can be a number representing the id, multiple numbers seperated by commas or the string 'all'.
   * @returns A Promise<AxiosResponse<object>> containing the returned data from the API.
   */
  getAlbum(id: number | string): Promise<AxiosResponse<ApiData>> {
    return this.get('album', id);
  }

  /**
   * A function to get one, multiple or all artists.
   * @param id The id of the artist. It can be a number representing the id, multiple numbers seperated by commas or the string 'all'.
   * @returns A Promise<AxiosResponse<object>> containing the returned data from the API.
   */
  getArtist(id: number | string): Promise<AxiosResponse<ApiData>> {
    return this.get('artist', id);
  }

  /**
   * A function to get all albums from the API.
   * @returns A Promise<AxiosResponse<object>> containing the returned data from the API.
   */
  getAllAlbums(): Promise<AxiosResponse<ApiData>> {
    return this.get('album', 'all');
  }

  /**
   * A function to get all artists from the API.
   * @returns A Promise<AxiosResponse<object>> containing the returned data from the API.
   */
  getAllArtists(): Promise<AxiosResponse<ApiData>> {
    return this.get('artist', 'all');
  }

  search(query: string): Promise<AxiosResponse<ApiData>> {
    return this.axiosInstance.get(`${this.apiUrl}/search/${query}`);
  }

  download(id: string): Promise<AxiosResponse<ApiData>> {
    return this.axiosInstance.get(`${this.apiUrl}/download/${id}`);
  }

  downloadqueue(): Promise<AxiosResponse<ApiData>> {
    return this.axiosInstance.get(`${this.apiUrl}/downloadqueue`);
  }

  videoInfo(id: string): Promise<AxiosResponse<ApiData>> {
    return this.axiosInstance.get(`${this.apiUrl}/info/${id}`);
  }

  update(database: string, index: number, field: string, value: number | string | boolean): Promise<AxiosResponse<ApiData>> {
    return this.axiosInstance.get(`${this.apiUrl}/update/${database}/${index}/${field}/${value}`);
  }

  insert(database: string): Promise<AxiosResponse<ApiData>> {
    return this.axiosInstance.get(`${this.apiUrl}/insert/${database}`);
  }
}
